/*
   gaineravr project:
   -- pepper, tiny gainer --
   main.c
   copyright (C) 2008, 2009, 2010 morecat_lab
   based on V-USB
*/

#include <string.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "usbdrv.h"
#include "oddebug.h"
#include "gio.h"

#if UART_CFG_HAVE_USART
#define HW_CDC_BULK_OUT_SIZE     8
#else
#define HW_CDC_BULK_OUT_SIZE     1
#endif
#define HW_CDC_BULK_IN_SIZE      8
/* Size of bulk transfer packets. The standard demands 8 bytes, but we may
 * be better off with less. Try smaller values if the communication hangs.
 */

enum {
    SEND_ENCAPSULATED_COMMAND = 0,
    GET_ENCAPSULATED_RESPONSE,
    SET_COMM_FEATURE,
    GET_COMM_FEATURE,
    CLEAR_COMM_FEATURE,
    SET_LINE_CODING = 0x20,
    GET_LINE_CODING,
    SET_CONTROL_LINE_STATE,
    SEND_BREAK
};


static PROGMEM char deviceDescrCDC[] = {    /* USB device descriptor */
    18,         /* sizeof(usbDescriptorDevice): length of descriptor in bytes */
    USBDESCR_DEVICE,        /* descriptor type */
    0x01, 0x01,             /* USB version supported */
    0x02,                   /* device class: CDC */
    0,                      /* subclass */
    0,                      /* protocol */
    8,                      /* max packet size */
    USB_CFG_VENDOR_ID,      /* 2 bytes */
    0xe1, 0x05,             /* 2 bytes: shared PID for CDC-ACM devices */
    USB_CFG_DEVICE_VERSION, /* 2 bytes */
    1,                      /* manufacturer string index */
    2,                      /* product string index */
#ifdef USE_SERIAL_NUMBER
    3,                      /* serial number string index */
#else
    0,
#endif
    1,                      /* number of configurations */
};

static PROGMEM char configDescrCDC[] = {   /* USB configuration descriptor */
    9,          /* sizeof(usbDescrConfig): length of descriptor in bytes */
    USBDESCR_CONFIG,    /* descriptor type */
    67,
    0,          /* total length of data returned (including inlined descriptors) */
    2,          /* number of interfaces in this configuration */
    1,          /* index of this configuration */
    0,          /* configuration name string index */
#if USB_CFG_IS_SELF_POWERED
    USBATTR_SELFPOWER,  /* attributes */
#else
    0x80, /* USBATTR_BUSPOWER,   attributes */
#endif
    USB_CFG_MAX_BUS_POWER/2,            /* max USB current in 2mA units */

    /* interface descriptor follows inline: */
    9,          /* sizeof(usbDescrInterface): length of descriptor in bytes */
    USBDESCR_INTERFACE, /* descriptor type */
    0,          /* index of this interface */
    0,          /* alternate setting for this interface */
    USB_CFG_HAVE_INTRIN_ENDPOINT,   /* endpoints excl 0: number of endpoint descriptors to follow */
    USB_CFG_INTERFACE_CLASS,
    USB_CFG_INTERFACE_SUBCLASS,
    USB_CFG_INTERFACE_PROTOCOL,
    0,          /* string index for interface */

    /* CDC Class-Specific descriptor */
    5,           /* sizeof(usbDescrCDC_HeaderFn): length of descriptor in bytes */
    0x24,        /* descriptor type */
    0,           /* header functional descriptor */
    0x10, 0x01,

    4,           /* sizeof(usbDescrCDC_AcmFn): length of descriptor in bytes */
    0x24,        /* descriptor type */
    2,           /* abstract control management functional descriptor */
    0x02,        /* SET_LINE_CODING,    GET_LINE_CODING, SET_CONTROL_LINE_STATE    */

    5,           /* sizeof(usbDescrCDC_UnionFn): length of descriptor in bytes */
    0x24,        /* descriptor type */
    6,           /* union functional descriptor */
    0,           /* CDC_COMM_INTF_ID */
    1,           /* CDC_DATA_INTF_ID */

    5,           /* sizeof(usbDescrCDC_CallMgtFn): length of descriptor in bytes */
    0x24,        /* descriptor type */
    1,           /* call management functional descriptor */
    3,           /* allow management on data interface, handles call management by itself */
    1,           /* CDC_DATA_INTF_ID */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x83,        /* IN endpoint number 3 */
    0x03,        /* attrib: Interrupt endpoint */
    8, 0,        /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL,        /* in ms */

    /* Interface Descriptor  */
    9,           /* sizeof(usbDescrInterface): length of descriptor in bytes */
    USBDESCR_INTERFACE,           /* descriptor type */
    1,           /* index of this interface */
    0,           /* alternate setting for this interface */
    2,           /* endpoints excl 0: number of endpoint descriptors to follow */
    0x0A,        /* Data Interface Class Codes */
    0,
    0,           /* Data Interface Class Protocol Codes */
    0,           /* string index for interface */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x01,        /* OUT endpoint number 1 */
    0x02,        /* attrib: Bulk endpoint */
    HW_CDC_BULK_OUT_SIZE, 0,        /* maximum packet size */
    0,           /* in ms */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x81,        /* IN endpoint number 1 */
    0x02,        /* attrib: Bulk endpoint */
    HW_CDC_BULK_IN_SIZE, 0,        /* maximum packet size */
    0,           /* in ms */
};


uchar usbFunctionDescriptor(usbRequest_t *rq)
{
  if(rq->wValue.bytes[1] == USBDESCR_DEVICE){
    usbMsgPtr = (uchar *)deviceDescrCDC;
    return sizeof(deviceDescrCDC);
  }else{  /* must be config descriptor */
    usbMsgPtr = (uchar *)configDescrCDC;
    return sizeof(configDescrCDC);
  }
}


static uchar requestType;
static uchar modeBuffer[7];
static uchar sendEmptyFrame;
static uchar intr3Status;    /* used to control interrupt endpoint transmissions */

static uchar buf[8];
static uchar endFlag = ENDFLAG_CONT;

uchar adVals[4];
int16_t pollTiming;
static uint16_t servo_wait = SERVO_POLLING_WAIT_LOOP; /* counter for servo timing */

#define POLL_INTERVAL_FACTOR 20		/* interval factor */

/* EEPROM VARIABLES */
static uchar __attribute__((section(".eeprom"))) calVal;
static uchar __attribute__((section(".eeprom"))) pollTimingVal = 150; /* 3000 is default for loop factor */

/* ----------------------------- command parser ----------------------------- */

#define toHex(i) (((i) <= 9)?('0' +(i)):((i)+'@'-9))

uchar *ucharToHex2(uchar data, uchar *s) { /* 数字->１６進文字（２桁） */
  uchar d;
  d = data >> 4;
  *s++ = toHex(d);
  d = data & 0x0f;
  *s++ = toHex(d);
  *s = '\0';
  return s;
}

uchar *uintToHex4(uint16_t data, uchar *s) { /* 数字->１６進文字（４桁） */
  uchar d = data >> 8;
  s = ucharToHex2(d, s);
  d = data & 0xff;
  return ucharToHex2(d, s);
}

uchar hexToUchar(uchar s) {	/* １６進文字->数字 */
  if (s >= '0' && s <= '9') {
    return (s - '0');
  }
  if (s >= 'A' && s <= 'F')  {
    return (s - 'A' + 10);
  }
  if (s >= 'a' && s <= 'f') {
    return (s - 'a' + 10);
  }
  return 0xff;			/* エラーの場合0xffを返す */
}

uchar hexToUchar2(uchar *s) {
  return (hexToUchar(*s) << 4) + hexToUchar(*(s+1));
}

uint16_t hexToInt16(uchar *s) {
  return (hexToUchar2(s) << 8) + hexToUchar2(s+2);
}

#define reportError() gioRxBufAppendString("!*")
#define reportVersion() gioRxBufAppendString(FIRMWARE_VERSION)
#define reportEcho(s) gioRxBufAppendString(s)
#define reportEchoChar(c) gioRxBufAppend(c)

#define reportDIN(c) {\
    buf[0] = c;\
    uintToHex4(gioGetDIPortAll(), buf+1);\
    buf[5] = '*';\
    buf[6] = '\0';\
    gioRxBufAppendString(buf); }

#define reportAIN(v) {	\
    ucharToHex2(v, buf);		\
    gioRxBufAppendString(buf); }

/* parse "KONFIGURATION_X" */
uchar parseKonfiguration(uchar *s) {
  uchar c;
  if (strncmp_P(s, PSTR("KONFIGURATION_"),14) != 0) {
    return 0;
  }
  switch (c = *(s+14)) {
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
    return (c - '0');
  }
  return 0;
}

void parseInput(uchar *s) {
  uchar c, n;
  switch (*s) {
    /**** reboot */
  case 'Q':
    if (pollMode != POLL_NONE) {
      endFlag = ENDFLAG_QUIT;
      return;
    } else {
      gioInit(0);			/* initialize */
    }
    break;

    /**** configuration */
  case 'K':			
    if ((c = parseKonfiguration(s)) > 0) {
      gioPortInit(c);
    } else {
      reportError();
      return;
    }
    break;

    /**** degital out (ALL) */
  case 'D':
    if ((config == 3) || (config == 4) || (config == 6) || (config == 8)) {
      gioSetDOPortAll(hexToInt16(s+1));
      break;
    }
    reportError();
    return;

    /**** degital out HIGH */
  case 'H':			
  case 'L':			
    if ((config == 3) || (config == 4) || (config == 6) || (config == 8)) {
      c = hexToUchar(*(s+1));
      if ((c < 2) || ((config == 6) && (c < 4))) {
	gioSetDOPort(c, (*s == 'H') ? 1 : 0);
	break;
      }
    }
    reportError();
    return;

    /**** degital in all ( we don't have d-in port in pepper board ) */
  case 'R':
    if ((config == 4) || (config == 5)) {
      reportDIN('R');
      return;
    }
    reportError();
    return;

    /**** degital in all (continuous mode) */
  case 'r':
    if ((config == 4) || (config == 5)) {
      pollMode = POLL_DIGITAL;
      endFlag = ENDFLAG_CONT;
      return;
    }
    reportError();
    return;

    /**** exit continunou mode */
  case 'E':
    if (pollMode != POLL_NONE) {
      endFlag = ENDFLAG_END;
      return;
    }
    reportError();
    return;

  case 'A':			/* analog out (ALL) Axx..xx*/
    if ((config == 1) || (config == 2)) {
      gioSetAOPortAll(s+1);
      break;
    }
    reportError();
    return;

  case 'a':			/* analog out (channel) anxx*/
    n = hexToUchar(*(s+1));
    c = hexToUchar2(s+2);
    if (((config != 1) && (config != 2)) || (n >= gioAOchannel())) {
      reportError();
      return;
    }
    gioSetAOPort(n, c);
    break;


  case 'P':			/* servo out (ALL) */
    if ((config == 7) || (config == 8)) {
      gioSetServoPortAll(s+1);
      break;
    }
    reportError();
    return;

  case 'p':			/* servo out (one channel) */
    n = hexToUchar(*(s+1));
    c = hexToUchar2(s+2);
    if (((config != 7) && (config != 8)) ||(n >= gioServochannel())) {
      reportError();
      return;
    }
    gioSetServoPort(n, c);
    break;

  case 'I':			/* analog in (ALL) I -> ixx..xx */
    if ((config == 1) || (config ==2) || (config == 3) || (config == 7)) {
      pollMode = POLL_ANALOG_ONCE;
      gioStartAd(0);
      return;
    }
    reportError();
    return;

  case 'i':			/* analog in (continuous mode) i -> ixx..xx */
    if ((config == 1) || (config ==2) || (config == 3) || (config == 7)) {
      pollMode = POLL_ANALOG_CONT;
      endFlag = ENDFLAG_CONT;
      gioStartAd(0);
      return;
    }
    reportError();
    return;

  case 'S':			/* analog in (channel, continuous mode) Sxx */
    if ((config == 1) || (config == 2) || (config == 3) || (config == 7)) {
      if (((c = hexToUchar(*(s+1))) == 0xff) ||  (c >= gioADchannel())) {
	reportError();
	return;
      }
      pollMode = POLL_ANALOG_1CH;
      endFlag = ENDFLAG_CONT;
      convertingCh = 0;
      gioStartAd(c);
      return;
    }
    reportError();
    return;

  case 'M':			/* adjust timing for poll mode */
    c = hexToUchar2(s+1);
    pollTiming = c * POLL_INTERVAL_FACTOR;
    eeprom_busy_wait();	/* EEPROMがレディになるまで待つ */
    eeprom_write_byte(&pollTimingVal, c); /* EEPROMに書き込む */

    break;

    /**** version number */
  case '?':
    reportVersion();
    return;

  default:
    reportError();
    return;
  }
  /* when the command is ok, we report the results */
  reportEcho(s);
}


/* ----------------------------- USB interface ----------------------------- */


uchar usbFunctionSetup(uchar data[8]) {
  usbRequest_t    *rq = (void *)data;

  if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
    if( rq->bRequest==GET_LINE_CODING || rq->bRequest==SET_LINE_CODING ){
      requestType = rq->bRequest;
      return 0xff;
      /*    GET_LINE_CODING -> usbFunctionRead()    */
      /*    SET_LINE_CODING -> usbFunctionWrite()    */
    }
    if(rq->bRequest == SET_CONTROL_LINE_STATE){
      /* Report serial state (carrier detect). On several Unix platforms,
       * tty devices can only be opened when carrier detect is set.
       */
      if( intr3Status==0 )
	intr3Status = 2;
    }
    /*  Prepare bulk-in endpoint to respond to early termination   */
    if((rq->bmRequestType & USBRQ_DIR_MASK) == USBRQ_DIR_HOST_TO_DEVICE)
      sendEmptyFrame  = 1;
  }
  return 0;
}

uchar usbFunctionRead( uchar *data, uchar len ) {

  if(requestType == GET_LINE_CODING){
    memcpy( data, modeBuffer, 7 );
    return 7;
  }
  return 0;   /* error -> terminate transfer */
}

uchar usbFunctionWrite( uchar *data, uchar len ) {

  if(requestType == SET_LINE_CODING){
    memcpy( modeBuffer, data, 7 );
    return 1;
  }
  return 1;   /* error -> accept everything until end */
}

void usbFunctionWriteOut( uchar *data, uchar len ) {
  uchar finish = 0;
  /*  usb -> rs232c:  transmit char    */
  for( ; len; len-- ) {
    if (*data == '*') {
      finish = 1;
    }
    gioTxBufAppend(*data++);
  }

  /* ここでTxBufferを解釈する*/
  if (finish == 1) {
    gioTxBufAppend('\0');
    parseInput(tx_buf);
    /* gioRxBufAppendString("\r\n"); */
    uwptr = 0;
  }
  /*  postpone receiving next data    */
  if( gioTxBytesFree()<=8 )
    usbDisableAllRequests();
}

static void hardwareInit(void) {
  uchar    i, j;

  /* activate pull-ups except on USB lines */
  USB_CFG_IOPORT   = (uchar)~((1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT));
  /* all pins input except USB (-> USB reset) */
#ifdef USB_CFG_PULLUP_IOPORT    /* use usbDeviceConnect()/usbDeviceDisconnect() if available */
  USBDDR    = 0;    /* we do RESET by deactivating pullup */
  usbDeviceDisconnect();
#else
  USBDDR    = (1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT);
#endif

  j = 0;
  while(--j){          /* USB Reset by device only required on Watchdog Reset */
    i = 0;
    while(--i);      /* delay >10ms for USB reset */
  }
#ifdef USB_CFG_PULLUP_IOPORT
  usbDeviceConnect();
#else
  USBDDR    = 0;      /*  remove USB reset condition */
#endif
}


/* ------------------------------------------------------------------------- */
/* ------------------------ Oscillator Calibration ------------------------- */
/* ------------------------------------------------------------------------- */

/* Calibrate the RC oscillator to 8.25 MHz. The core clock of 16.5 MHz is
 * derived from the 66 MHz peripheral clock by dividing. Our timing reference
 * is the Start Of Frame signal (a single SE0 bit) available immediately after
 * a USB RESET. We first do a binary search for the OSCCAL value and then
 * optimize this value with a neighboorhod search.
 * This algorithm may also be used to calibrate the RC oscillator directly to
 * 12 MHz (no PLL involved, can therefore be used on almost ALL AVRs), but this
 * is wide outside the spec for the OSCCAL value and the required precision for
 * the 12 MHz clock! Use the RC oscillator calibrated to 12 MHz for
 * experimental purposes only!
 */
static void calibrateOscillator(void) {
  uchar step = 128;
  uchar trialValue = 0, optimumValue;
  int   x, optimumDev, targetValue = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);

  /* do a binary search: */
  do{
    OSCCAL = trialValue + step;
    x = usbMeasureFrameLength();    /* proportional to current real frequency */
    if(x < targetValue)             /* frequency still too low */
      trialValue += step;
    step >>= 1;
  }while(step > 0);
  /* We have a precision of +/- 1 for optimum OSCCAL here */
  /* now do a neighborhood search for optimum value */
  optimumValue = trialValue;
  optimumDev = x; /* this is certainly far away from optimum */
  for(OSCCAL = trialValue - 1; OSCCAL <= trialValue + 1; OSCCAL++){
    x = usbMeasureFrameLength() - targetValue;
    if(x < 0)
      x = -x;
    if(x < optimumDev){
      optimumDev = x;
      optimumValue = OSCCAL;
    }
  }
  OSCCAL = optimumValue;
}
/*
Note: This calibration algorithm may try OSCCAL values of up to 192 even if
the optimum value is far below 192. It may therefore exceed the allowed clock
frequency of the CPU in low voltage designs!
You may replace this search algorithm with any other algorithm you like if
you have additional constraints such as a maximum CPU clock.
For version 5.x RC oscillators (those with a split range of 2x128 steps, e.g.
ATTiny25, ATTiny45, ATTiny85), it may be useful to search for the optimum in
both regions.
*/

void    usbEventResetReady(void) {
  calibrateOscillator();
  eeprom_write_byte(&calVal, OSCCAL);   /* store the calibrated value in EEPROM */
}

/* Polling function for gio */
void gioPoll() {
  int16_t val;


  static int cnt = 0;
  /* no need for polling */
  if (pollMode == POLL_NONE) {
    return;
  }

  /* Digital in */
  if (pollMode == POLL_DIGITAL) {
    if (endFlag == ENDFLAG_END) {
      pollMode = POLL_NONE;
      reportEcho("E*");
    } else if (endFlag == ENDFLAG_QUIT) {
      pollMode = POLL_NONE;
      gioInit(0);
      reportEcho("Q*");
    } else if ((--cnt) == 0) {
      reportDIN('r');
      cnt = pollTiming;
    }
    return;
  }

  /* Analog in */
  if (pollMode == POLL_ANALOG_WAIT) { /* iコマンドで次のAD開始待ち */
    if ((--cnt) == 0) {
      convertingCh = 0;
      /* ADを再スタートする */
      setupCH(convertingCh);
      wait(2);
      ADCStart();
      pollMode = POLL_ANALOG_CONT;
    }
    return;
  }

  if (pollMode == POLL_ANALOG_WAIT1) { /* Sコマンドで次のAD開始待ち */
    if ((--cnt) == 0) {
      ADCStart();
      pollMode = POLL_ANALOG_1CH;
    }
    return;
  }

  if ((val = gioPollAd()) >= 0) { /* 変換終了時のみ */
    if (convertingCh == 3) {
      val =- 200;		/* 200LSBを引くだけにする */
    } else {
      val = val >> 2;		/* 10bit -> 8bit */
    }
    adVals[(uchar)convertingCh] = val;	/* 値を記録する */

    if (pollMode == POLL_ANALOG_1CH) { /* Sコマンドの場合 1チャンネルのスキャン */
      reportEchoChar('S');
      reportAIN(val);
      reportEchoChar('*');
      if (endFlag == ENDFLAG_END) {
	pollMode = POLL_NONE;
	reportEcho("E*");
      } else if (endFlag == ENDFLAG_QUIT) {
	pollMode = POLL_NONE;
	gioInit(0);
	reportEcho("Q*");
      } else {
	cnt = pollTiming;
	pollMode = POLL_ANALOG_WAIT1;
      }
      return;
    }

    if (pollMode == POLL_ANALOG_ONCE) { /* Iコマンドの場合 */
      convertingCh++;
      if (convertingCh >= gioADchannel()) { /* 全チャンネル終了 */
	pollMode = POLL_NONE;
	convertingCh = CONVERTING_CH_NONE;
	gioStopAd();
	reportADValues('I');
      } else {
	setupCH(convertingCh);
	wait(2);
	ADCStart();
      }
      return;
    }

    if (pollMode == POLL_ANALOG_CONT) {
      /* 次のチャンネルに設定し、開始する */
      convertingCh++;
      if (convertingCh >= gioADchannel()) { /* 全チャンネル終了 */
	reportADValues('i');
	if (endFlag == ENDFLAG_END) {
	  pollMode = POLL_NONE;
	  reportEcho("E*");
	} else if (endFlag == ENDFLAG_QUIT) {
	  pollMode = POLL_NONE;
	  gioInit(0);
	  reportEcho("Q*");
	} else {
	  cnt = pollTiming;
	  pollMode = POLL_ANALOG_WAIT;
	}
      } else {
	setupCH(convertingCh);
	wait(2);
	ADCStart();
      }
      return;
    }
  }
}

void reportADValues(uchar cmd) {
  uchar i;
  reportEchoChar(cmd);
  for (i = 0 ; i < 4 ; i++) {
    reportAIN(adVals[i]);
  }
  reportEchoChar('*');
}

uchar aOutValue [SERVO_MAX] = { 0, 255 };
#define SERVO_SET_VALUE(ch, val) { aOutValue[ch] = val; }
#define SERVO_OUT_MIN0 44
#define SERVO_OUT_MIN1 44
#define SERVO_CONV_OCR0(v) (SERVO_OUT_MIN0 + (((v) / 10) *4))
#define SERVO_CONV_OCR1(v) (SERVO_OUT_MIN1 + (((v) / 10) *4))

void servoPoll() {
  if ((config != 7) && (config != 8)) {
    return;
  }
  if (--servo_wait == 0) {

    TCCR0A = 0b11000000; /* Normal mode(0) , SET OC0A on compare match   */
    TCCR0B = 0b10000000; /* TIMER STOP/ force compare match OC0A */
    TCCR0A = 0b10000000; /* Normal mode(0), CLEAR OC0A on compare match */
    OCR0A = SERVO_CONV_OCR0(aOutValue[0]);
    TCNT0 = 0; /* clear counter */
    TCCR0B = 0b00000100; /* Normal mode(0) 1/256 clock / START Timer*/
    
    TCCR1 = 0b00000000;		/* Timer STOP */
    GTCCR = 0b00111000;		/* set OC1B on compare match, force reset OC1B  */
    GTCCR = 0b00100000;		/* clear OC1B on compare match */
    OCR1B = SERVO_CONV_OCR1(aOutValue[1]);
    TCNT1 = 0;			/* clear timer */
    TCCR1 = 0b00001001;		/* Prescaler 1/256 / START timer */

    servo_wait = SERVO_POLLING_WAIT_LOOP;
  }
}

int main(void) {
  uchar   i;
  uchar   calibrationValue;

  calibrationValue = eeprom_read_byte(&calVal); /* calibration value from last time */
  if(calibrationValue != 0xff){
    OSCCAL = calibrationValue;
  }
  odDebugInit();
  usbDeviceDisconnect();
  for(i=0;i<20;i++){  /* 300 ms disconnect */
    _delay_ms(15);
  }

  /* configモードをEEPROMより読み込んで設定する */
  eeprom_busy_wait();
  pollTiming = eeprom_read_byte(&pollTimingVal) * POLL_INTERVAL_FACTOR;

  wdt_enable(WDTO_1S);
  hardwareInit();
  usbInit();
  gioInit(0);

  intr3Status = 0;
  sendEmptyFrame  = 0;

  sei();

  for(;;){    /* main event loop */
    wdt_reset();
    usbPoll();
    gioPoll(); /* (for continuous mode ) */

    servoPoll();		/* for servo control */
    /*    gio -> usb:  transmit char        */
    if( usbInterruptIsReady() ) {
      uchar bytesRead, *data;
      
      bytesRead = gioRxBytesAvailable(&data);
      if(bytesRead > 0 || sendEmptyFrame){
	if(bytesRead >= HW_CDC_BULK_IN_SIZE) {
	  bytesRead = HW_CDC_BULK_IN_SIZE;
	  /* send an empty block after last data block to indicate transfer end */
	  sendEmptyFrame  = 1;
	}
	else
	  sendEmptyFrame  = 0;
	usbSetInterrupt(data, bytesRead);
	gioRxDidReadBytes(bytesRead);
      }
    }

    /* We need to report rx and tx carrier after open attempt */
    if(intr3Status != 0 && usbInterruptIsReady3()){
      static uchar serialStateNotification[10] = {0xa1, 0x20, 0, 0, 0, 0, 2, 0, 3, 0};
      
      if(intr3Status == 2){
	usbSetInterrupt3(serialStateNotification, 8);
      }else{
	usbSetInterrupt3(serialStateNotification+8, 2);
      }
      intr3Status--;
    }
  }
  return 0;
}

/* EOF */
